import datetime as dt

def third_tuesday_of_month(year, month):
  d = dt.datetime(year, month, 1)
  wd = d.weekday()
  # monday
  if wd == 0:
    return 16
  # tuesday
  if wd == 1:
    return 15
  # wensday
  if wd == 2:
    return 14
  # thursday
  if wd == 3:
    return 13
  # friday
  if wd == 4:
    return 12
  # saturday
  if wd == 5:
    return 11
  # sunday 6
  if wd == 6:
    return 17


import numpy as np
import pandas as pd
import ib_insync
from ib_insync import *
import sqlite3
import wansuite as ws






def downloadstock(sql,table,enddate,duration):
    """
    enddate=2023"+"0210"+" 19:00:00"
    duration="60 D"
    """
    

    util.startLoop()
    ib = IB()
    ib.connect('localhost', 7496, clientId=np.random.randint(10000))
    conn=sqlite3.connect(sql,timeout=10)
    cursor=conn.cursor()
    stocklist=ws.wsql.getPossibleValue(sql,table,"ticker")
    misslist=[]
    for n in range(len(stocklist)): # #for stock in 
        if n%500==0:
            print(n)
        try: 
            stock=stocklist[n]
            st=Stock(stock,"SMART","USD")
            bars = ib.reqHistoricalData(st, 
                            endDateTime=endate,
                            durationStr=duration,
                            barSizeSetting="5 mins",
                            whatToShow="TRADES", #TRADES
                            useRTH=0,
                            keepUpToDate=False,                    
                            formatDate=1)
            for dt in bars:
                cursor.execute(""" 
                    INSERT INTO equity (datetime,ticker, open, close, high, low, volume, barsize ) 
                    VALUES(?,?,?,?,?,?,?,?)""",(str(dt.date),stock,dt.open,dt.close,dt.high,dt.low,dt.volume,dt.barCount))
        except:
            print("cannot download",st)
            misslist.append(stock)
    return misslist
    conn.commit()
    conn.close()


import pandas as pd
from sqlalchemy import create_engine,Table, Column, Integer, String, MetaData,Date,DateTime,Time
import mysql.connector
import numpy as np

def listBase():
    address='mysql+mysqlconnector://root:tiger1006@localhost/'
    engine = create_engine(address)
    # execute query to get all databases
    result_set = engine.execute("SHOW DATABASES")

    # print the list of databases
    for row in result_set:
        print(row[0])

def listTable(sql):
    address='mysql+mysqlconnector://root:tiger1006@localhost:3306/'+sql
    engine = create_engine(address)
    # get the metadata of the database
    metadata = MetaData(bind=engine)

# reflect the database and retrieve the table names
    metadata.reflect()
    table_names = metadata.tables.keys()

    # print the table names
    for table_name in table_names:
        print(table_name)

def createBase(sql):
    address='mysql+mysqlconnector://root:tiger1006@localhost/'
    engine = create_engine(address)
    engine.execute("CREATE DATABASE "+sql)
def createTable(sql,table,col_list,type_list):
    address='mysql+mysqlconnector://root:tiger1006@localhost:3306/'+sql
    engine = create_engine(address)
    # Create a metadata object that will hold the definition of the table
    metadata = MetaData()
    my_table=Table(table,metadata,Column(col_list[0],type_list[0]))
    
    if len(col_list)>1:
        for i in range(1,len(col_list)):
            new_col=Column(col_list[i],type_list[i])
            my_table.append_column(new_col)
    
    metadata.create_all(engine)
    
  

def deleteBase(sql):
    address='mysql+mysqlconnector://root:tiger1006@localhost:3306/'+sql
    engine = create_engine(address)
    metadata = MetaData()

# associate the metadata with the engine
    metadata.bind = engine

# drop all tables associated with the metadata
    metadata.drop_all(bind=engine)
    
    
def deleteTable(sql,table):
    address='mysql+mysqlconnector://root:tiger1006@localhost:3306/'+sql
    engine = create_engine(address)

    connection = engine.connect()

    table_name = table
    query = f"DROP TABLE IF EXISTS {table_name};"
    connection.execute(query)

    connection.close()
    
    


def df2table(sql,df,table,flag,chunk=10000):
    '''
    chunk is the size for batch save. by default 10000
    -------
    flag:  
    -------
    we're using 'replace' to replace the existing table with the new data.
    You can also use 'append' to add the new data to the existing table 
    or 'fail' to raise an error if the table already exists.
    '''
    address='mysql+mysqlconnector://root:tiger1006@localhost:3306/'+sql
    engine = create_engine(address)
    df.fillna(value='NULL',inplace=True)
    
    df.to_sql(table, con=engine, chunksize=chunk,if_exists=flag, index=False)

def table2df(sql,table,condition=None):
    '''
    condition: "country = 'us'"
    table:  "investing"
    sql: ""macro""
    '''
    if condition==None:
        address='mysql+mysqlconnector://root:tiger1006@localhost:3306/'+sql
        engine = create_engine(address)
        query = "SELECT * FROM "+ table
        df = pd.read_sql(query, engine)

        return df.replace(to_replace='NULL',value=np.nan)

    else:
        address='mysql+mysqlconnector://root:tiger1006@localhost:3306/'+sql
        engine = create_engine(address)
        query = "SELECT * FROM "+ table+" WHERE "+condition
        df = pd.read_sql(query, engine)

        return df.replace(to_replace='NULL',value=np.nan)



 


import sqlite3
import time 
from bs4 import BeautifulSoup
from helium import *
import pandas as pd
from . import msql
def idfromurl(url):
    id_url=None
    for n in range(10):
        if url[-n]!="-":
            id_url=url[-n:]
        else:
            break
    return id_url

    
def updateToday(newlink,sql,res):
    update=pd.DataFrame(columns=["index","time",	"country","indicator","value","forecast"])
    resource=msql.table2df(sql,res).drop_duplicates(subset="link").set_index("link")
    for l in newlink:
        cou=resource.loc[l,"country"]
        ind=resource.loc[l,"indicator"]
        url=l
        url_id=idfromurl(url)
        driver=start_chrome(url,headless=True)
        #driver.find_element("id", "showMoreHistory"+url_id).click()
        if url=="https://www.investing.com/economic-calendar/interest-rate-decision-168":
            for n in range(2):
                try:

                    driver.find_element("id", "showMoreHistory"+url_id).click()
                    time.sleep(1)
                    #driver.find_element_by_link_text("Show more").click()
                except:
                    break
      
        html_source=driver.page_source
        data=pd.read_html(html_source)[0].set_index("Release Date")[["Time","Actual","Forecast"]]
        #kill_browser()
        driver.quit()

       
        
        
        newind=[]
        for l in data.index:
            l=str(l)
            if l[-1]==")":
                newind.append(l[:-5])
            else:
                newind.append(l)
        data.index=newind
        data.index=pd.to_datetime(data.index).date
        data=data.sort_index()
        data=data.reset_index().drop_duplicates(subset=["index"]).set_index("index")
        if str(data.iloc[0]["Actual"])[-1]=="K":
            val0=[]
            for al0 in data["Actual"].dropna():
                if al0[0]=="-":
                    val0.append(-float(al0.replace("K","").replace("-","").replace(",",""))*1000)
                else:
                    val0.append(float(al0.replace("K","").replace(",",""))*1000)
            data.loc[data["Actual"].dropna().index,"Actual"]=val0   
            val0=[]
            for al0 in data["Forecast"].dropna():
                if al0[0]=="-":
                    val0.append(-float(al0.replace("K","").replace("-","").replace(",",""))*1000)
                else:
                    val0.append(float(al0.replace("K","").replace(",",""))*1000)
            data.loc[data["Forecast"].dropna().index,"Forecast"]=val0    
        elif str(data.iloc[0]["Actual"])[-1]=="M":
            val0=[]
            for al0 in data["Actual"].dropna():
                if al0[0]=="-":
                    val0.append(-float(al0.replace("M","").replace("-","").replace(",",""))*1000000)
                else:
                    val0.append(float(al0.replace("M","").replace(",",""))*1000000)
            data.loc[data["Actual"].dropna().index,"Actual"]=val0   
            val0=[]
            for al0 in data["Forecast"].dropna():
                if al0[0]=="-":
                    val0.append(-float(al0.replace("M","").replace("-","").replace(",",""))*1000000)
                else:
                    val0.append(float(al0.replace("M","").replace(",",""))*1000000)
            data.loc[data["Forecast"].dropna().index,"Forecast"]=val0    
        elif str(data.iloc[0]["Actual"])[-1]=="B":
            val0=[]
            for al0 in data["Actual"].dropna():
                if al0[0]=="-":
                    val0.append(-float(al0.replace("B","").replace("-","").replace(",",""))*1000000000)
                else:
                    val0.append(float(al0.replace("B","").replace(",",""))*1000000000)
            data.loc[data["Actual"].dropna().index,"Actual"]=val0   
            val0=[]
            for al0 in data["Forecast"].dropna():
                if al0[0]=="-":
                    val0.append(-float(al0.replace("B","").replace("-","").replace(",",""))*1000000000)
                else:
                    val0.append(float(al0.replace("B","").replace(",",""))*1000000000)
            data.loc[data["Forecast"].dropna().index,"Forecast"]=val0  
        elif str(data.iloc[0]["Actual"])[-1]=="T":
            val0=[]
            for al0 in data["Actual"].dropna():
                if al0[0]=="-":
                    val0.append(-float(al0.replace("T","").replace("-","").replace(",",""))*1000000000000)
                else:
                    val0.append(float(al0.replace("T","").replace(",",""))*1000000000000)
            data.loc[data["Actual"].dropna().index,"Actual"]=val0   
            val0=[]
            for al0 in data["Forecast"].dropna():
                if al0[0]=="-":
                    val0.append(-float(al0.replace("T","").replace("-","").replace(",",""))*1000000000000)
                else:
                    val0.append(float(al0.replace("T","").replace(",",""))*1000000000000)
            data.loc[data["Forecast"].dropna().index,"Forecast"]=val0  
        elif str(data.iloc[0]["Actual"])[-1]=="%":
            val0=[]
            for al0 in data["Actual"].dropna():
                if al0[0]=="-":
                    val0.append(-float(al0.replace("%","").replace("-","").replace(",",""))/100)
                else:
                    val0.append(float(al0.replace("%","").replace(",",""))/100)
            data.loc[data["Actual"].dropna().index,"Actual"]=val0   
            val0=[]
            for al0 in data["Forecast"].dropna():
                if al0[0]=="-":
                    val0.append(-float(al0.replace("%","").replace("-","").replace(",",""))/100)
                else:
                    val0.append(float(al0.replace("%","").replace(",",""))/100)
            data.loc[data["Forecast"].dropna().index,"Forecast"]=val0   
        newdate=data["Actual"].dropna().index[-1]
        update.loc[len(update)]=[str(newdate),data.loc[newdate,"Time"],cou,ind, data.loc[newdate,"Actual"],data.loc[newdate,"Forecast"]]
        print(newdate,cou,ind)
    return update
    
def addNewFeature(addlist,sql,table):
    '''
    addlist=[["us","Featurename",link],[]]
    save to sql.table
    '''
    newfeature=pd.DataFrame(columns=["country","indicator","link"])
    for l in addlist:
        newfeature.loc[len(newfeature)]=l
    msql.df2table(sql,newfeature,table,"append")

    
    
def buildNewFeature(addlist):
    
    '''
    new
    addlist=[["us","Featurename",link],[]]
    output: new data point
    '''
   
    new=pd.DataFrame(columns=["index","time","country","indicator","value","forecast"])
    for l in addlist:
        url=l[2]
        cou=l[0]
        indi=l[1]
        url_id=idfromurl(url)
        driver=start_chrome(url,headless=True)
        for n in range(200):
            try:

                driver.find_element("id", "showMoreHistory"+url_id).click()
                time.sleep(1)
                #driver.find_element_by_link_text("Show more").click()
            except:
                break
        print(indi,n)
        html_source=driver.page_source
        kill_browser()
        #driver.quit()
        data=pd.read_html(html_source)[0].set_index("Release Date")[["Time","Actual","Forecast"]]


        newind=[]
        for l in data.index:
            l=str(l)
            if l[-1]==")":
                newind.append(l[:-5])
            else:
                newind.append(l)
        data.index=newind
        data.index=pd.to_datetime(data.index)
        data=data.sort_index().drop_duplicates().rename_axis("date").reset_index().drop_duplicates(subset="date").set_index("date")
        if str(data.iloc[0]["Actual"])[-1]=="K":
            val0=[]
            for al0 in data["Actual"].dropna():
                if al0[0]=="-":
                    val0.append(-float(al0.replace("K","").replace("-","").replace(",",""))*1000)
                else:
                    val0.append(float(al0.replace("K","").replace(",",""))*1000)
            data.loc[data["Actual"].dropna().index,"Actual"]=val0   
            val0=[]
            for al0 in data["Forecast"].dropna():
                if al0[0]=="-":
                    val0.append(-float(al0.replace("K","").replace("-","").replace(",",""))*1000)
                else:
                    val0.append(float(al0.replace("K","").replace(",",""))*1000)
            data.loc[data["Forecast"].dropna().index,"Forecast"]=val0    
        elif str(data.iloc[0]["Actual"])[-1]=="M":
            val0=[]
            for al0 in data["Actual"].dropna():
                if al0[0]=="-":
                    val0.append(-float(al0.replace("M","").replace("-","").replace(",",""))*1000000)
                else:
                    val0.append(float(al0.replace("M","").replace(",",""))*1000000)
            data.loc[data["Actual"].dropna().index,"Actual"]=val0   
            val0=[]
            for al0 in data["Forecast"].dropna():
                if al0[0]=="-":
                    val0.append(-float(al0.replace("M","").replace("-","").replace(",",""))*1000000)
                else:
                    val0.append(float(al0.replace("M","").replace(",",""))*1000000)
            data.loc[data["Forecast"].dropna().index,"Forecast"]=val0    
        elif str(data.iloc[0]["Actual"])[-1]=="B":
            val0=[]
            for al0 in data["Actual"].dropna():
                if al0[0]=="-":
                    val0.append(-float(al0.replace("B","").replace("-","").replace(",",""))*1000000000)
                else:
                    val0.append(float(al0.replace("B","").replace(",",""))*1000000000)
            data.loc[data["Actual"].dropna().index,"Actual"]=val0   
            val0=[]
            for al0 in data["Forecast"].dropna():
                if al0[0]=="-":
                    val0.append(-float(al0.replace("B","").replace("-","").replace(",",""))*1000000000)
                else:
                    val0.append(float(al0.replace("B","").replace(",",""))*1000000000)
            data.loc[data["Forecast"].dropna().index,"Forecast"]=val0  
        elif str(data.iloc[0]["Actual"])[-1]=="T":
            val0=[]
            for al0 in data["Actual"].dropna():
                if al0[0]=="-":
                    val0.append(-float(al0.replace("T","").replace("-","").replace(",",""))*1000000000000)
                else:
                    val0.append(float(al0.replace("T","").replace(",",""))*1000000000000)
            data.loc[data["Actual"].dropna().index,"Actual"]=val0   
            val0=[]
            for al0 in data["Forecast"].dropna():
                if al0[0]=="-":
                    val0.append(-float(al0.replace("T","").replace("-","").replace(",",""))*1000000000000)
                else:
                    val0.append(float(al0.replace("T","").replace(",",""))*1000000000000)
            data.loc[data["Forecast"].dropna().index,"Forecast"]=val0   
        elif "%" in str(data["Actual"].dropna().iloc[-1]):
            val0=[]
            for al0 in data["Actual"].dropna():
                if "-" in al0:
                    val0.append(-float(al0.replace("%","").replace("-","").replace(",",""))/100)
                else:
                    val0.append(float(al0.replace("%","").replace(",",""))/100)
            data.loc[data["Actual"].dropna().index,"Actual"]=val0   
            val0=[]
            for al0 in data["Forecast"].dropna():
                if al0[0]=="-":
                    val0.append(-float(al0.replace("%","").replace("-","").replace(",",""))/100)
                else:
                    val0.append(float(al0.replace("%","").replace(",",""))/100)
            data.loc[data["Forecast"].dropna().index,"Forecast"]=val0   

        for d in data.index:
            da=str(d)
            t=str(data.loc[d,"Time"])
            v=float(data.loc[d,"Actual"])
            p=float(data.loc[d,"Forecast"])

            new.loc[len(new)]=[da,t,cou,indi,v,p]
    return new    
def updateTableDaily(table,linklist):
    '''
    table: name of table to save data "investing"
    linklist: link we need to update to table
    '''
    
    for n in linklist:
        try:
            update=updateToday([n],"macro","resource")
            msql.df2table("macro",update,table,"append")
        except:
            print("The following link does not work")
            print(n)
            pass
        
        
        
def getLink(lockkey):
    url="https://www.investing.com/economic-calendar/"
    driver=start_chrome(url,headless=True)
    elem = driver.find_elements_by_xpath("//*[@href]")
    driver=start_chrome(url,headless=True)
    
    newlink=[]
    for el in elem:
        a=el.get_attribute("href")
        if a[:43]=='https://www.investing.com/economic-calendar' and a!='https://www.investing.com/economic-calendar/' and a!= 'https://www.investing.com/economic-calendar/#':
            block=False
            for k in lockkey:
                if k in a:
                    block=True
                    break
            if block==False:
                newlink.append(a)
    addlist=[]
    resource=ws.msql.table2df("macro","resource").drop_duplicates().set_index("link")
    for l in newlink:
        if l not in resource.index:
            addlist.append(["","",l])
    return newlink,addlist



import numpy as np
import pandas as pd
import sqlite3

def newTable(sql,new_table,struct):
    """
    -------------
    Example : struct: "(
       country TEXT,
       indicator TEXT,
       link TEXT,
       netdate TEXT
    )"
    """
    #Connecting to sqlite
    conn = sqlite3.connect(sql)

    cursor = conn.cursor()
   
    sql ='''CREATE TABLE {new_table}'''+struct
    cursor.execute(sql)
    print("Table created successfully........")

    # Commit your changes in the database
    conn.commit()

    #Closing the connection
    conn.close()


def fetchallTables(sql):
    conn = sqlite3.connect(sql)
    # Get all table names
    cursor = conn.cursor()
    cursor.execute("SELECT name FROM sqlite_master WHERE type='table';")
    table_names = cursor.fetchall()
    return table_names

#Print columns names and types of each column for one of sql
def table_col(sql,table):
    conn = sqlite3.connect(sql)
    cursor = conn.cursor()

# Get the names of all columns in the data table
    cursor.execute("PRAGMA table_info("+table+")")
    columns = [row[1] for row in cursor.fetchall()]
    cursor.execute("PRAGMA table_info("+table+")")
    types = [row[2] for row in cursor.fetchall()]
    #print(columns)
    #print(types)

# Close the connection
    conn.close()
    return columns, types

def deleteData(sql,table,condition):
    """
    Parameters
    -------------
    - condition: i.e. "indicator='RetailSalesMoMForecast' and country='us'"
    
    Examples
    -----------------------------
    deleteData(sql,table,c "indicator='RetailSalesMoMForecast' and country='us'"):
    """
    conn = sqlite3.connect(sql)
    cursor = conn.cursor()
    statement="DELETE FROM "+table+" WHERE "+condition

# delete rows where the value in column 'other_column' is less than 10
    cursor.execute(statement)

    conn.commit()
    conn.close()
    
    
def replaceValue(sql, table,col_name,old,new):
    conn = sqlite3.connect(sql)
    cursor = conn.cursor()
    statement="UPDATE "+table+" SET "+col_name+"="+str(new)+" WHERE "+col_name+"="+str(old)
    # Replace the value of column "column_name" in the table "table_name" where the value of column "condition_column" is "condition_value" with "new_value"
    cursor.execute(statement)

    conn.commit()
    conn.close()
    
def readSql(sql,table):
    conn = sqlite3.connect(sql)
    return pd.read_sql_query("SELECT * from %s" % table, conn)
    
def getPossibleValue(sql,table,col):
    conn = sqlite3.connect(sql)
    cursor = conn.cursor()

    cursor.execute(f"SELECT DISTINCT {col} FROM {table}")
    values = cursor.fetchall()
    namelist=[]
    for value in values:
        namelist.append(value[0])
    
    conn.close()
    return namelist

def batchInsert(sql,table,df,col):
    conn=sqlite3.connect(sql)
    cursor=conn.cursor()
    for l in series.index:
        t=str(ppi.loc[l,"Time"])
        d=str(l.date())
        v=ppi.loc[l,"Forecast"]
        cursor.execute(""" 
            INSERT INTO indicators (date, time, country, indicator, value) 
            VALUES(?, ? ,"us","CPICoreYoYForecast",?)""",(d,t,v))
    conn.commit()
    conn.close()